%% 
clear all;
clc;
%% Parameters
%l is the maximum number of the detected droplets
    l=40000;
%limitstk is the Stokes number limit set for the filtering method
    limitstk=0.1;
%measurementrange is the minimum size the PDA measurement in micrometers
    measurementrange=0.5145;
%Gas dynamic viscosity [Pa*s]
    mu=18.691*10^(-6); 
% fluid density [kg/m3]
    rho=838.8;
% flow characteristic length scale, where d_{h} is the hydraulic diameter
% of the nozzle in [mm]
    dh=0.4;
    cl=30*dh*0.001;

%% Load data
currentFolder = pwd;
allFiles = dir (currentFolder);
allNames={allFiles.name};
allNames(contains(allNames,'.txt')==0)=[];
allNames(contains(allNames,'README.txt')==1)=[];
allNamesConverted=char(allNames);

                M = fileread(allNamesConverted);
                M = strrep(M, ',', '.');
                FID = fopen(allNamesConverted, 'w');
                fwrite(FID, M, 'char');
                fclose(FID);
                M = dlmread(allNamesConverted,'' ,6, 0);
                prob=length(M)+1;
                    for j=prob:l
                    M(j,:)=zeros;
                    end  

      %D is the droplet size
                D = M(:,8);
      %vax is the axial velocity component
                vax = M(:,4); 
      %vrad is the radial velocity component
                vrad = M(:,5); 
      %v is the total velocity
                v=sqrt(vrad.^2+vax.^2);
                       

%% Iteration and data processing
%Remove droplets with D lower than "measurementrange"
            prefilt=[D,v];
            prefilt( prefilt(:,1)<measurementrange,:)=[];
%Outlier fitering by droplet size
             prefilt= prefilt';
           [TF] = isoutlier( prefilt(1,:));
             prefilt(:, TF==1) = [];
             prefilt= prefilt';
             Dbase=prefilt(:,1);
             vbase=prefilt(:,2);

% 1st step of the iteration: mean velocity is equal to the mean droplet
% velocity, calculation of Stk
    if length(prefilt(:,2))==1
        vmean_a(1)=prefilt(1,2);
    else
        vmean_a(1)=mean(prefilt(:,2));  
    end
        Stkbase=(prefilt(:,1)).^2.*abs(vmean_a(1)-prefilt(:,2))*10^(-12)*rho/mu/18/cl;
         
    matrix=[prefilt(:,1),prefilt(:,2),Stkbase];         
    matrix2=matrix;
    matrix2(:,end+1)=1:length(matrix(:,1));
          
    it=1;
%iteration from 2nd step  
  while vmean_a(it)~=0
          
        goodelements=matrix2;
        delelements=matrix2;
        %Droplets over Stk limit is selected in delelements
        delelements(delelements(:,3)<=limitstk,:)=[];
        %Smallest droplet which doesn't fit the Stk condition: maxdelD
        maxdelD=min(delelements(:,1));                
            if isempty(maxdelD)==1
                break        
            end
        Dcrit(it)= maxdelD; %Critical droplet diameter
                   
        %Droplets fitting the Stk condition:
        goodelements(goodelements(:,3)>limitstk,:)=[];
        %Droplets with diameter below the critical droplet diameter
        goodelements(goodelements(:,1)>maxdelD,:)=[];

            if length(goodelements(:,2))==1
                vmean_a(it+1)=goodelements(1,2);
            else
                vmean_a(it+1)= mean(goodelements(:,2));  
            end       
        Stkfiltered=(goodelements(:,1)).^2.*abs(vmean_a(it+1)-goodelements(:,2))*10^(-12)*rho/mu/18/cl;
        matrix2=[goodelements(:,1),goodelements(:,2),Stkfiltered,goodelements(:,4)];
        it=it+1;
  end

%%OFiltered data's mean values
if matrix2(:,1)==0
    Dmeanfilt=0;
    vmeanfilt=0;
    Stkmeanfilt=0;
    elementsfilt=0;

    Dfilt(1:length(matrix2))=0;
    vfilt(1:length(matrix2))=0;
    Stkfilt(1:length(matrix2))=0;
else
    Dmeanfilt=mean(matrix2(:,1));
    vmeanfilt=mean(matrix2(:,2));
    Stkmeanfilt=mean(matrix2(:,3));
    elementsfilt=length(matrix2(:,1));
    Dfilt(1:length(matrix2(:,1)))=matrix2(:,1);
    vfilt(1:length(matrix2(:,2)))=matrix2(:,2);
    Stkfilt(1:length(matrix2(:,3)))=matrix2(:,3);
end

Dfilt=Dfilt';
vfilt=vfilt';
Stkfilt=Stkfilt';


%%Original data's mean values
if matrix(:,1)==1
    Dmean=0;
    vmean=0;
    Stkmean=0;
    elements=0;
else
    Dmean=mean(matrix(:,1));
    vmean=mean(matrix(:,2));
    Stkmean=mean(matrix(:,3));
    elements=length(matrix(:,1));
end   
    

%% Save results
resulttable(2:3,1)=["Base data";"Filtered data"];
resulttable(1,2:6)=["Mean droplet size";"Mean velocity";"Mean Stokes number";"data #";"Dcrit"];
resulttable(2:3,2:5)=[Dmean,vmean,Stkmean,elements;...
                        Dmeanfilt,vmeanfilt,Stkmeanfilt,elementsfilt];
resulttable(3,6)=Dcrit;
save('resulttable.mat','resulttable','-v7.3')
datafilt=struct('Dfilt',Dfilt,'vfilt',vfilt,'Stkfilt',Stkfilt);
save('datafilt.mat','datafilt','-v7.3')
database=struct('Stkbase',Stkbase,'Dbase',Dbase,'vbase',vbase,'allNamesConverted',allNamesConverted);
save('database.mat','database','-v7.3')